// Constants
global.IRCPath = argument0;
global.IRCPacketId = argument1;

// IRC server settings
global.IRCHost = 'irc.esper.net';
global.IRCPort = 6667;
global.IRCChan = '#gg2';
global.IRCUser = 'GG2';
global.IRCRealName = 'Gang Garrison 2 IRC plugin';

// Variables

// Scripts
global.specialCsvToList = '
// Splits CSV into items
// argument0 - string comma-separated values
// argument1 - delimeter
// return value - ds_list containing strings of values

var list, text, delimeter;
text = argument0;
delimeter = argument1;
list = ds_list_create();

while (string_pos(delimeter, text) != 0)
{
    ds_list_add(list, string_copy(text, 1, string_pos(delimeter,text) - 1));
    text = string_copy(text, string_pos(delimeter, text) + string_length(delimeter), string_length(text) - string_pos(delimeter, text));
}
ds_list_add(list, text);

return list;
';

global.IRCFixNickAndSend = "
// Make sanitised nick
var override;
override = argument0;

var nick, safeNick, i, c;
nick = global.playerName;
safeNick = '';
for (i = 1; i <= min(string_length(nick), 30); i += 1) {
    c = string_char_at(nick, i);
    // [a-zA-Z0-9]
    if (('a' <= c && c <= 'z') || ('A' <= c && c <= 'Z') || ('0' <= c && c <= '9')) {
        // IRC has no problems with these characters, append to IRC nickname
        safeNick += c;
    } else {
        // IRC might have problems with this character, append an underscore instead
        safeNick += '_';
    }
}

if (safeNick != nick or override) {
    // Update our GG2 name to this nick
    global.playerName = safeNick;
    gg2_write_ini('Settings', 'PlayerName', safeNick);
    write_ubyte(global.serverSocket, PLAYER_CHANGENAME);
    write_ubyte(global.serverSocket, string_length(safeNick));
    write_string(global.serverSocket, safeNick);
    socket_send(global.serverSocket);
}
with (global.IRCClient) {
    write_string(IRCSocket, 'NICK ' + global.playerName + CRLF);
    socket_send(IRCSocket);
}
";

global.IRCPrePadString = "
// prePadString(string, num, char)
var string, num, char;
string = argument0;
num = argument1;
char = argument2;

while (string_length(string) < num) {
    string = char + string;
}

return string;
"

// Objects
global.IRCClient = object_add();
object_set_persistent(global.IRCClient, 1);
object_event_add(global.IRCClient, ev_create, 0, '
    IRCSocket = tcp_connect(global.IRCHost, global.IRCPort);
    IRCLog = ds_list_create();
    CRLF = chr(13) + chr(10);
    buf = "";

    var now;
    now = date_current_datetime();
    startDate = string(date_get_year(now))
                + "-" + execute_string(global.IRCPrePadString, string(date_get_month(now)), 2, "0")
                + "-" + execute_string(global.IRCPrePadString, string(date_get_day(now)), 2, "0")
                + "T" + execute_string(global.IRCPrePadString, string(date_get_hour(now)), 2, "0")
                + "-" + execute_string(global.IRCPrePadString, string(date_get_minute(now)), 2, "0")
                + "-" + execute_string(global.IRCPrePadString, string(date_get_second(now)), 2, "0")
                + "Z";

    execute_string(global.IRCFixNickAndSend);
    write_string(IRCSocket, "USER " + global.IRCUser + " 0 * :" + global.IRCRealName + CRLF);
    socket_send(IRCSocket);
');
object_event_add(global.IRCClient, ev_step, ev_step_begin, '
    if (socket_has_error(IRCSocket)) {
        show_message("Socket error: " + socket_error(IRCSocket));
        game_end();
        exit;
    }
    var available;
    available = tcp_receive_available(IRCSocket);

    buf += read_string(IRCSocket, available);
    
    var lines, i;
    lines = execute_string(global.specialCsvToList, buf, CRLF);
    for (i = 0; i < ds_list_size(lines); i += 1) {
        if (i == ds_list_size(lines) - 1) {
            buf = ds_list_find_value(lines, i);
        } else {
            var parts;
            parts = execute_string(global.specialCsvToList, ds_list_find_value(lines, i), " ");
            var prefix;
            prefix = "";
            // Remove prefix
            if (string_char_at(ds_list_find_value(parts, 0), 1) == ":") {
                prefix = ds_list_find_value(parts, 0);
                ds_list_delete(parts, 0);
            }
            var type;
            type = ds_list_find_value(parts, 0);
            switch (type) {
            case "PING":
                var token;
                token = ds_list_find_value(parts, 1);
                if (string_char_at(token, 1) == ":") {
                    token = string_copy(token, 2, string_length(token) - 1);
                }
                write_string(IRCSocket, "PONG " + token + CRLF);
                write_string(IRCSocket, "PONG " + ds_list_find_value(parts, 1) + CRLF);
                socket_send(IRCSocket);
            break;
            case "NOTICE":
            // RPL_MOTD
            case "372":
                var line;
                line = "* ";
                var j;
                for (j = 2; j < ds_list_size(parts); j += 1) {
                    var part;
                    part = ds_list_find_value(parts, j);
                    if (string_char_at(part, 1) == ":" and j == 2) {
                        line += string_copy(part, 2, string_length(part) - 1) + " ";
                    } else {
                        line += part + " ";
                    }
                }
                ds_list_add(IRCLog, line);
            break;
            // RPL_MOTDSTART
            case "375":
                ds_list_add(IRCLog, "<MOTD>");
            break;
            // RPL_ENDOFMOTD
            case "376":
                ds_list_add(IRCLog, "</MOTD>");
                write_string(IRCSocket, "JOIN " + global.IRCChan + CRLF);
                socket_send(IRCSocket);
            break;
            // RPL_TOPIC
            case "332":
                var line;
                line = "";
                var j;
                for (j = 3; j < ds_list_size(parts); j += 1) {
                    var part;
                    part = ds_list_find_value(parts, j);
                    if (string_char_at(part, 1) == ":" and j == 3) {
                        line += string_copy(part, 2, string_length(part) - 1) + " ";
                    } else {
                        line += part + " ";
                    }
                }
                ds_list_add(IRCLog, "* Topic for " + ds_list_find_value(parts, 2) + ": " + line);
            break;
            // Seems to be "set by" for topic
            case "333":
                ds_list_add(IRCLog, "* Topic for " + ds_list_find_value(parts, 2) + " set by " + ds_list_find_value(parts, 3));
            break;
            case "JOIN":
                ds_list_add(IRCLog, "* " + string_copy(prefix, 2, string_length(prefix) - 1) + " joined " + ds_list_find_value(parts, 1));
            break;
            case "MODE":
                var modes;
                // Channel modes
                if (ds_list_find_value(parts, 1) == global.IRCChan) {
                    prefix = string_copy(prefix, 2, string_length(prefix) - 1);
                    var nickBits;
                    nickBits = execute_string(global.specialCsvToList, prefix, "!");
                    var line;
                    line = "";
                    var j;
                    for (j = 2; j < ds_list_size(parts); j += 1) {
                        var part;
                        part = ds_list_find_value(parts, j);
                        if (string_char_at(part, 1) == ":" and j == 2) {
                            line += string_copy(part, 2, string_length(part) - 1) + " ";
                        } else {
                            line += part + " ";
                        }
                    }
                    ds_list_add(IRCLog, "* " + ds_list_find_value(nickBits, 0) + " sets mode: " + line);
                    ds_list_destroy(nickBits);
                } else {
                    modes = ds_list_find_value(parts, 2);
                    string_copy(modes, 2, string_length(modes) - 1);
                    ds_list_add(IRCLog, "* Your mode was set to mode " + modes);
                }
            break;
            case "PRIVMSG":
                prefix = string_copy(prefix, 2, string_length(prefix) - 1);
                var nickBits;
                nickBits = execute_string(global.specialCsvToList, prefix, "!");
                var line;
                line = "";
                var j;
                for (j = 2; j < ds_list_size(parts); j += 1) {
                    var part;
                    part = ds_list_find_value(parts, j);
                    if (string_char_at(part, 1) == ":" and j == 2) {
                        line += string_copy(part, 2, string_length(part) - 1) + " ";
                    } else {
                        line += part + " ";
                    }
                }
                // /me :D
                if (string_copy(line, 1, 8) == chr(1) + "ACTION ") {
                    line = string_copy(line, 9, string_length(line) - 8);
                    // Channel or private message?
                    if (ds_list_find_value(parts, 1) == global.IRCChan) {
                        ds_list_add(IRCLog, "!* " + ds_list_find_value(nickBits, 0) + " " +line);
                    } else {
                        ds_list_add(IRCLog, "(PRIVMSG) !* " + ds_list_find_value(nickBits, 0) + " " + line);
                    }
                } else {
                    // Channel or private message?
                    if (ds_list_find_value(parts, 1) == global.IRCChan) {
                        ds_list_add(IRCLog, "<" + ds_list_find_value(nickBits, 0) + "> " + line);
                    } else {
                        ds_list_add(IRCLog, "(PRIVMSG) <" + ds_list_find_value(nickBits, 0) + "> " + line);
                    }
                }
                ds_list_destroy(nickBits);
            break;
            case "QUIT":
                prefix = string_copy(prefix, 2, string_length(prefix) - 1);
                var nickBits;
                nickBits = execute_string(global.specialCsvToList, prefix, "!");
                var line;
                line = "";
                var j;
                for (j = 1; j < ds_list_size(parts); j += 1) {
                    var part;
                    part = ds_list_find_value(parts, j);
                    if (string_char_at(part, 1) == ":" and j == 1) {
                        line += string_copy(part, 2, string_length(part) - 1) + " ";
                    } else {
                        line += part + " ";
                    }
                }
                ds_list_add(IRCLog, "* " + ds_list_find_value(nickBits, 0) + " quit (" + line + ")");
                ds_list_destroy(nickBits);
            break;
            case "PART":
                prefix = string_copy(prefix, 2, string_length(prefix) - 1);
                var nickBits;
                nickBits = execute_string(global.specialCsvToList, prefix, "!");
                var line;
                line = "";
                var j;
                for (j = 2; j < ds_list_size(parts); j += 1) {
                    var part;
                    part = ds_list_find_value(parts, j);
                    if (string_char_at(part, 1) == ":" and j == 2) {
                        line += string_copy(part, 2, string_length(part) - 1) + " ";
                    } else {
                        line += part + " ";
                    }
                }
                ds_list_add(IRCLog, "* " + ds_list_find_value(nickBits, 0) + " left " + ds_list_find_value(parts, 1) + " (" + line + ")");
                ds_list_destroy(nickBits);
            break;
            case "NICK":
                prefix = string_copy(prefix, 2, string_length(prefix) - 1);
                var nickBits;
                nickBits = execute_string(global.specialCsvToList, prefix, "!");
                var newNick;
                newNick = ds_list_find_value(parts, 1);
                newNick = string_copy(newNick, 2, string_length(newNick) - 1);
                ds_list_add(IRCLog, "* " + ds_list_find_value(nickBits, 0) + " is now known as " + newNick);
                // It is my name that is being changed!
                if (string_lower(ds_list_find_value(nickBits, 0)) == string_lower(global.playerName)) {
                    global.playerName = newNick;
                    execute_string(global.IRCFixNickAndSend, 1);
                }
                ds_list_destroy(nickBits);
            break;
            // RPL_NAMREPLY
            case "353":
            // RPL_ENDOFNAMES
            case "366":
                // Skip
            break;
            default:
                ds_list_add(IRCLog, "? " + ds_list_find_value(lines, i));
            break;
            }
            ds_list_destroy(parts);
        }
    }
    ds_list_destroy(lines);
');
object_event_add(global.IRCClient, ev_other, ev_game_end, '
    if (!directory_exists(working_directory + "\IRCLogs")) {
        directory_create(working_directory + "\IRCLogs");
    }
    var fp, i;
    fp = file_text_open_write(working_directory + "\IRCLogs\" + startDate + ".txt");
    for (i = 0; i < ds_list_size(IRCLog); i += 1) {
        file_text_write_string(fp, ds_list_find_value(IRCLog, i));
        file_text_writeln(fp);
    }
    file_text_close(fp);
');

global.IRCWindow = object_add();
object_set_depth(global.IRCWindow, -110002);
object_event_add(global.IRCWindow, ev_create, 0, '
    open = false;
    showback = true;
');
object_event_add(global.IRCWindow, ev_step, ev_step_end, '
    if ((!instance_exists(InGameMenuController) && !instance_exists(OptionsController) && !instance_exists(ControlsController) && !instance_exists(HUDOptionsController)) || instance_exists(global.cheatycheat)) {
        // "open" to enter text
        if (!open && keyboard_check_pressed(ord("Y"))) {
            open = true;
            keyboard_string = "";
            instance_create(0, 0, global.cheatycheat);
        // toggle background
        } else if (!open && keyboard_check_pressed(ord("T"))) {
            showback = !showback;
        // Send message and "close"
        } else if (open && keyboard_check_pressed(vk_enter)) {
            with (global.cheatycheat) {
                instance_destroy();
            }
            open = false;
            // Command
            if (string_char_at(keyboard_string, 1) == "/") {
                if (string_copy(keyboard_string, 1, 5) == "/msg ") {
                    keyboard_string = string_copy(keyboard_string, 6, string_length(keyboard_string) - 5);
                    var nick, msg;
                    nick = string_copy(keyboard_string, 1, string_pos(" ", keyboard_string) - 1);
                    msg = string_copy(keyboard_string, string_pos(" ", keyboard_string) + 1, string_length(keyboard_string) - string_pos(" ", keyboard_string));
                    with (global.IRCClient) {
                        write_string(IRCSocket, "PRIVMSG " + nick + " :" + msg + CRLF);
                        socket_send(IRCSocket);
                        ds_list_add(IRCLog, "(PRIVMSG->" + nick + ") <" + global.playerName + "> " + msg);
                    }
                } else if (string_copy(keyboard_string, 1, 6) == "/nick ") {
                    global.playerName = string_copy(keyboard_string, 7, string_length(keyboard_string) - 6);
                    execute_string(global.IRCFixNickAndSend, 1);
                } else if (string_copy(keyboard_string, 1, 4) == "/me ") {
                    with (global.IRCClient) {
                        keyboard_string = string_copy(keyboard_string, 5, string_length(keyboard_string) - 4);
                        write_string(IRCSocket, "PRIVMSG " + global.IRCChan + " :" + chr(1) + "ACTION " + keyboard_string + CRLF);
                        socket_send(IRCSocket);
                        ds_list_add(IRCLog, "!* " + global.playerName + " " + keyboard_string);
                    }
                } else {
                    with (global.IRCClient) {
                        ds_list_add(IRCLog, "Error: Unrecognised command");
                    }
                }
            } else if (string_length(keyboard_string) != 0) {
                with (global.IRCClient) {
                    write_string(IRCSocket, "PRIVMSG " + global.IRCChan + " :" + keyboard_string + CRLF);
                    socket_send(IRCSocket);
                    ds_list_add(IRCLog, "<" + global.playerName + "> " + keyboard_string);
                }
            }
            keyboard_string = "";
        }
    }
');
object_event_add(global.IRCWindow, ev_draw, 0, '
    var xoffset, yoffset, xsize, ysize, xmax, ymax;
    xoffset = view_xview[0];
    yoffset = view_yview[0];
    xsize = view_wview[0];
    ysize = view_hview[0];
    xmax = 380;
    ymax = 180;
    x = view_xview[0]+7;
    y = view_yview[0]+ysize-80;

    if (showback) {
        draw_set_alpha(0.2928932188134525);
        draw_set_color(c_black);
        draw_rectangle(x, y, x + xmax, y - ymax, false);
        draw_set_alpha(0.2928932188134525 * 0.2928932188134525);
        draw_set_color(c_black);
        draw_rectangle(x, y, x + xmax, y - ymax, true);
    }
    
    draw_set_halign(fa_left);
    draw_set_valign(fa_bottom);
    draw_set_alpha(1);
    var _y, i, line;
    _y = 0;
    // Text entry
    if (open) {
        draw_set_color(c_orange);
        line = "> " + string_replace(keyboard_string, "#", "\#");
        draw_text(x, y, line);
        _y += string_height(line);
    }
    draw_set_color(c_white);
    for (i = ds_list_size(global.IRCClient.IRCLog) - 1; i >= 0; i -= 1) {
        line = string_replace(ds_list_find_value(global.IRCClient.IRCLog, i), "#", "\#");
        if (_y + string_height_ext(line, -1, xmax) > ymax) {
            break;
        }
        // Dark green for info lines
        if (string_char_at(line, 1) == "*") {
            draw_set_color(c_green);
        // /me lines
        } else if (string_char_at(line, 1) == "!") {
            draw_set_color(c_fuchsia);
        // ? lines
        } else if (string_char_at(line, 1) == "?") {
            draw_set_color(c_maroon);
        } else {
            draw_set_color(c_white);
        }
        draw_text_ext(x, y - _y, line, -1, xmax);
        _y += string_height_ext(line, -1, xmax);
    }
');

// Add events to Player to keep nickname safe
object_event_add(Player, ev_step, ev_step_begin, "
    if (!variable_local_exists('oldName')) {
        oldName = '';
    }
    if (global.myself == id and name != oldName) {
        oldName = name;
        
        execute_string(global.IRCFixNickAndSend);
    }
");

// I STOLE THIS FROM LORGAN'S CHAT CODE SO GIVE HIM THE CREDIT
//create a cheaty cheat object to prevent you from moving and opening chat menus while typing :p
global.cheatycheat = object_add();
object_set_parent(global.cheatycheat,InGameMenuController);
object_event_add(global.cheatycheat,ev_step,ev_step_normal,'//no');
object_event_add(global.cheatycheat,ev_draw,0,'//no');
object_event_add(global.cheatycheat,ev_keypress,vk_enter,'//no');
object_event_add(global.cheatycheat,ev_keypress,vk_escape,'
    with(global.IRCWindow) open = false;
    instance_create(0,0,InGameMenuController);
    instance_destroy();
');

// Add event to PlayerControl to create client/window
object_event_add(PlayerControl, ev_step, ev_step_end, '
    if (!instance_exists(global.IRCClient)) {
        instance_create(0, 0, global.IRCClient);
    }
    if (!instance_exists(global.IRCWindow)) {
        instance_create(0, 0, global.IRCWindow);
    }
');
